# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_node_id_extract
:Synopsis:          This module is used by pytest to verify that Node IDs can be extracted successfully from URLs.
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     23 Apr 2020
:Version:           1.0.2
"""

import os
import sys

import pytest


def set_package_path():
    """This function adds the high-level khoros directory to the sys.path list."""
    sys.path.insert(0, os.path.abspath('../..'))
    return


def get_test_data():
    """This function retrieves the test data that will be used in the test functions.

    :returns: The ``test_data`` dictionary with the node types and associated test URLs
    """
    base_test_url = "https://community.khoros.com/t5"
    test_data = {
        'blog': f'{base_test_url}/Khoros-Now-Blog/bg-p/relnote',
        'board': f'{base_test_url}/Growing-Successful-Communities/bd-p/growingcommunities',
        'category': f'{base_test_url}/Forums/ct-p/Forums',
        'contest': f'{base_test_url}/Concert-Photo-Contest/con-p/vacationPhotoContest',
        'group': f'{base_test_url}/Khoros-Rockstars/gp-p/khorosRockstars',
        'idea': f'{base_test_url}/Big-Ideas/idb-p/bigIdeas',
        'message': f'{base_test_url}/Womens-Running-Shoes/Looking-for-an-idea-on-which-of-these-to-choose/m-p/997#M78',
        'qa': f'{base_test_url}/Ask-a-Baker/qa-p/bakingqanda',
        'tkb': f'{base_test_url}/Getting-Started/tkb-p/gettingStarted'
    }
    return test_data


def test_with_valid_node_types():
    """This function tests that Node IDs can be extracted from URLs when valid node types are given.

    :returns: None
    :raises: :py:exc:`AssertionError`, :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`,
             :py:exc:`khoros.errors.exceptions.NodeIDNotFoundError`,
             :py:exc:`khoros.errors.exceptions.NodeTypeNotFoundError`
    """
    # Import the base module
    set_package_path()
    from khoros.structures import nodes

    # Get the test data
    test_data = get_test_data()

    # Perform the test for each key value pair
    for node_type, url in test_data.items():
        node_id = nodes.get_node_id(url, node_type)
        assert (node_id is not False) and (len(node_id) != 0)
    return


def test_with_invalid_node_types():
    """This function tests to ensure that invalid node types will raise the appropriate exception.

    :returns: None
    :raises: :py:exc:`AssertionError`, :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`
    """
    # Import the base and exceptions modules
    set_package_path()
    from khoros.structures import nodes
    from khoros.errors import exceptions

    # Get the test data
    test_data = get_test_data()

    # Test passing a made-up node type
    with pytest.raises(exceptions.InvalidNodeTypeError):
        nodes.get_node_id(test_data.get('blog'), 'gonna_break')

    # Test passing the wrong node type for a given URL
    with pytest.raises(exceptions.InvalidNodeTypeError):
        nodes.get_node_id(test_data.get('group'), 'tkb')

    # Return when finished
    return


def test_with_only_url():
    """This function tests the :py:func:`khoros.objects.base.get_node_id` function when only a URL is passed.

    :returns: None
    :raises: :py:exc:`AssertionError`
    """
    # Import the base module
    set_package_path()
    from khoros.structures import nodes

    # Get the test data
    test_data = get_test_data().values()

    # Test getting the Node ID for each URL type
    for url in test_data:
        node_id = nodes.get_node_id(url)
        assert (node_id is not False) and (len(node_id) != 0)
    return


def test_url_without_node():
    """This function tests to ensure that an appropriate exception is raised when a URL does not contain a valid node.

    :returns: None
    :raises: :py:exc:`AssertionError`, :py:exc:`khoros.errors.exceptions.NodeTypeNotFoundError`
    """
    # Import the base and exceptions modules
    set_package_path()
    from khoros.structures import nodes
    from khoros.errors import exceptions

    # Test passing a URL that does not have a node within it
    with pytest.raises(exceptions.NodeTypeNotFoundError):
        nodes.get_node_id('https://community.khoros.com/this-is-a-test-url')
    return



# -*- coding: utf-8 -*-
"""
:Module:         khoros.utils.tests.test_library_import
:Synopsis:       This module is used by pytest to verify that the primary package can be imported successfully
:Created By:     Jeff Shurtliff
:Last Modified:  Jeff Shurtliff
:Modified Date:  31 Jan 2020
"""

import os
import sys


def set_package_path():
    """This function adds the high-level khoros directory to the sys.path list."""
    sys.path.insert(0, os.path.abspath('../..'))
    return


# Test importing the primary khoros package
def import_pkg_operation():
    """This function imports the primary package and returns ``True`` when successful."""
    import khoros
    return True


# Verify that the overall package can be successfully imported
def test_library_import():
    """This function tests to confirm that the primary package can be imported successfully."""
    set_package_path()
    assert import_pkg_operation() is True



# -*- coding: utf-8 -*-
"""
:Module:            khoros.objects.base
:Synopsis:          This module handles general (i.e. object-agnostic) functions relating to API objects.
:Usage:             ``from khoros.objects import base``
:Example:           ``node_id = base.get_node_id('https://community.khoros.com/t5/Khoros-Blog/bg-p/relnote', 'blog')``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     17 Jul 2020
"""

import warnings

from ..structures import nodes
from ..utils import log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def get_node_id(url, node_type=None):
    """This **deprecated** function retrieves the Node ID for a given node within a URL.

    .. deprecated:: 2.1.0
       Use :py:func:`khoros.structures.nodes.get_node_id` instead.

    :param url: The URL from which to parse out the Node ID
    :type url: str
    :param node_type: The node type (e.g. ``blog``, ``tkb``, ``message``, etc.) for the object in the URL
    :type node_type: str, NoneType
    :returns: The Node ID retrieved from the URL
    :raises: :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`,
             :py:exc:`khoros.errors.exceptions.NodeIDNotFoundError`,
             :py:exc:`khoros.errors.exceptions.NodeTypeNotFoundError`
    """
    warnings.warn("The 'khoros.objects.base.get_node_id' function has been deprecated. Use " +
                  "'khoros.structures.nodes.get_node_id' instead.", DeprecationWarning)
    return nodes.get_node_id(url, node_type)


def __get_node_type_identifier(_node_type_lookup):
    """This **deprecated** function attempts to identify the appropriate node type for a function.

    .. deprecated:: 2.1.0
       Use :py:func:`khoros.structures.nodes._get_node_type_identifier` instead.

    :param _node_type_lookup: The value to look up as a node type
    :type _node_type_lookup: str
    :returns: The appropriate node type (if found)
    :raises: :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`
    """
    warnings.warn("The 'khoros.objects.base.__get_node_type_identifier' function has been deprecated. Use " +
                  "'khoros.structures.nodes._get_node_type_identifier' instead.", DeprecationWarning)
    return nodes._get_node_type_identifier(_node_type_lookup)


def get_node_type_from_url(url):
    """This function attempts to retrieve a node type by analyzing a supplied URL.

    .. deprecated:: 2.1.0
       Use :py:func:`khoros.structures.nodes.get_node_type_from_url` instead.

    :param url: The URL from which to extract the node type
    :type url: str
    :returns: The node type based on the URL provided
    :raises: :py:exc:`khoros.errors.exceptions.NodeTypeNotFoundError`
    """
    warnings.warn("The 'khoros.objects.base.get_node_type_from_url' function has been deprecated. Use " +
                  "'khoros.structures.nodes.get_node_type_from_url' instead.", DeprecationWarning)
    return nodes.get_node_type_from_url(url)


class Mapping:
    """This class includes dictionaries that map API fields and values to those used in this SDK.

    .. deprecated:: 2.1.0
       Use :py:class:`khoros.structures.nodes.Mapping` instead.
    """
    warnings.warn("The 'khoros.objects.base.Mapping' class has been deprecated. Use " +
                  "'khoros.structures.nodes.Mapping' instead.", DeprecationWarning)
    node_url_mapping = nodes.Mapping.node_url_mapping
    proper_name_mapping = nodes.Mapping.proper_name_mapping



# -*- coding: utf-8 -*-
"""
:Module:            khoros.objects.roles
:Synopsis:          This module includes functions that handle roles and permissions.
:Usage:             ``from khoros.objects import roles``
:Example:           ``count = roles.get_total_role_count()``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     17 Jul 2020
"""

from .. import api, liql, errors
from ..utils import log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)

ROLE_TYPES = {
    'board': 'b',
    'category': 'c',
    'group_hub': 'g',
    'top_level': 't'
}


def get_total_role_count(khoros_object, return_dict=False, total=True, top_level=False, board=False, category=False,
                         group_hub=False):
    """This function retrieves the total role count for one or more role type(s).

    .. versionadded:: 2.4.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param return_dict: Determines if the data should be returned as a dictionary (``False`` by default)
    :type return_dict: bool
    :param total: Indicates that the total overall role count should be returned (``True`` by default)
    :type total: bool
    :param top_level: Indicates that the total top-level role count should be returned (``False`` by default)
    :type top_level: bool
    :param board: Indicates that the total board-level role count should be returned (``False`` by default)
    :type board: bool
    :param category: Indicates that the total category-level role count should be returned (``False`` by default)
    :type category: bool
    :param group_hub: Indicates that the total group hub-level role count should be returned (``False`` by default)
    :type group_hub: bool
    :returns: The role count(s) as an integer, tuple or dictionary, depending on the arguments supplied
    :raises: :py:exc:`khoros.errors.exceptions.InvalidRoleTypeError`
    """
    response = liql.perform_query(khoros_object, liql_query="SELECT id FROM roles", verify_success=True)
    counts = {
        'total': api.get_results_count(response)
    }
    if not total:
        del counts['total']
    if top_level or board or category or group_hub:
        roles_dict = api.get_items_list(response)
        count_types = {'top_level': top_level, 'board': board, 'category': category, 'group_hub': group_hub}
        for count_type, should_count in count_types.items():
            if should_count:
                counts[count_type] = count_role_types(count_type, roles_dict)
    if not return_dict:
        if len(counts) == 1:
            counts = counts.get(list(counts.keys())[0])
        else:
            counts = tuple(counts.values())
    return counts


def count_role_types(role_type, roles_dict):
    """This function returns the total count for a specific role type.

    .. versionadded:: 2.4.0

    :param role_type: The role type for which to return the count (e.g. ``board``, ``category``, etc.)
    :type role_type: str
    :param roles_dict: Dictionary of the roles for a given Khoros Community environment
    :type roles_dict: dict
    :returns: The total count for the role type as an integer
    :raises: :py:exc:`khoros.errors.exceptions.InvalidRoleTypeError`
    """
    if role_type not in ROLE_TYPES.keys() and role_type not in ROLE_TYPES.values():
        raise errors.exceptions.InvalidRoleTypeError(role_type=role_type)
    elif role_type in ROLE_TYPES:
        role_type = ROLE_TYPES.get(role_type)
    count = 0
    for role in roles_dict:
        if role['id'].startswith(f"{role_type}:"):
            count += 1
    return count


def get_roles_for_user(khoros_object, user_id):
    """This function returns all roles associated with a given User ID.

    .. versionadded:: 2.4.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param user_id: The User ID for which to retrieve the roles data
    :returns: A dictionary with data for each role associated with the given User ID
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`
    """
    response = liql.perform_query(khoros_object, liql_query=f"SELECT * FROM roles WHERE users.id = '{user_id}'",
                                  verify_success=True)
    return api.get_items_list(response)



# -*- coding: utf-8 -*-
"""
:Module:            khoros.objects.albums
:Synopsis:          This module includes functions that handle albums that contain images
:Usage:             ``from khoros.objects import albums``
:Example:           ``response = albums.create_album(khoros_obj, title='My Album', hidden=True)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     17 Jul 2020
"""

from .. import api, liql, errors
from ..utils import core_utils, log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def create(khoros_object, title=None, description=None, owner_id=None, hidden=False, default=False,
           full_response=False):
    """This function creates a new image album for a user.

    .. versionadded:: 2.3.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param title: The title of the album to be created
    :type title: str, None
    :param description: The description of the album
    :type description: str, None
    :param owner_id: The User ID of the album owner

                     .. note:: If not defined, the owner will be the user performing the API call.

    :type owner_id: str, int, None
    :param hidden: Defines if the album should be public (default) or hidden
    :type hidden: bool
    :param default: Defines if this will be the default album for the user (``False`` by default)
    :type default: bool
    :param full_response: Defines if the full response should be returned instead of the outcome (``False`` by default)
    :type full_response: bool
    :returns: Boolean value indicating a successful outcome (default) or the full API response
    """
    # TODO: Add functionality for "cover" field with "image" datatype
    album_json = format_album_json(title, description, owner_id, hidden, default)
    query_uri = f"{khoros_object.core['v2_base']}albums"
    response = api.post_request_with_retries(query_uri, album_json, khoros_object=khoros_object)
    result = api.query_successful(response)
    return response.json() if full_response else result


def format_album_json(title=None, description=None, owner_id=None, hidden=None, default=False):
    """This function formats the JSON payload for the API call.

    .. versionadded:: 2.3.0

    :param title: The title of the album to be created
    :type title: str, None
    :param description: The description of the album
    :type description: str, None
    :param owner_id: The User ID of the album owner

                     .. note:: If not defined, the owner will be the user performing the API call.

    :type owner_id: str, int, None
    :param hidden: Defines if the album should be public (default) or hidden
    :type hidden: bool
    :param default: Defines if this will be the default album for the user (``False`` by default)
    :type default: bool
    :returns: The JSON payload for the album API call
    """
    # TODO: Add functionality for "cover" field with "image" datatype
    privacy_level = {True: "private", False: "public"}
    album_json = {
        "data": {
            "type": "album",
            "title": _null_to_blank(title),
            "description": _null_to_blank(description),
            "owner": {
                "id": f"{owner_id}"
            },
            "privacy_level": privacy_level.get(hidden),
            "default": default
        }
    }
    return album_json


def _null_to_blank(_value):
    """This function returns a blank string when a null / NoneType value is passed.

    .. versionadded:: 2.3.0
    """
    return "" if _value is None else _value


def get_albums_for_user(khoros_object, user_id=None, login=None, public=None, private=None, verify_success=False,
                        allow_exceptions=True):
    """This function returns data for the albums owned by a given user.

    .. versionadded:: 2.3.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param user_id: The User ID for the album owner
    :type user_id: str, int
    :param login: The username of the album owner
    :type login: str
    :param public: Indicates that **public** albums should be returned (all albums returned by default)
    :type public: bool
    :param private: Indicates that **private** albums should be returned (all albums returned by default)
    :type private: bool
    :param verify_success: Optionally check to confirm that the API query was successful (``False`` by default)
    :type verify_success: bool
    :param allow_exceptions: Defines whether or not exceptions can be raised for responses returning errors

                             .. caution:: This does not apply to exceptions for missing required data.

    :type allow_exceptions: bool
    :returns: A list of dictionaries representing each album
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.MissingAuthDataError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    if not user_id and not login:
        raise errors.exceptions.MissingRequiredDataError("A 'user_id' or 'login' is required to retrieve data")
    query = "SELECT * FROM albums WHERE"
    if user_id:
        if not core_utils.is_numeric(user_id) and not login:
            raise errors.exceptions.MissingRequiredDataError("The 'user_id' must be numeric or a 'login' is needed")
        query = f"{query} owner.id = '{user_id}'"
    else:
        query = f"{query} owner.login = '{login}'"
    if not public or not private:
        if public:
            query = f"{query} AND privacy_level = 'public'"
        else:
            query = f"{query} AND privacy_level = 'hidden'"
    response = liql.perform_query(khoros_object, liql_query=query, verify_success=verify_success,
                                  allow_exceptions=allow_exceptions)
    return response
